@@ -29,6 +29,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
+#include <sys/param.h>
 #include <string.h>
 #include <netinet/in.h>
 #ifdef HAVE_LINUX_CDROM_H
@@ -92,7 +93,7 @@ typedef struct {
 #if defined (__linux__) || defined(__sun)
   struct cdrom_tochdr    tochdr;
   struct cdrom_tocentry  tocent[100];
-#elif defined (__FreeBSD_kernel__) || defined (__OpenBSD__)
+#elif defined (__FreeBSD_kernel__) || (defined(BSD) && BSD >= 199306)
   struct ioc_toc_header  tochdr;
   struct cd_toc_entry    *tocent;
   off_t                  cur_sec;
@@ -117,7 +118,7 @@ typedef struct {
 
   int                    cur_track;
 
-#if defined (__linux__) || defined(__sun) || defined (__FreeBSD_kernel__) || defined (__OpenBSD__)
+#if defined (__linux__) || defined(__sun) || defined (__FreeBSD_kernel__) || (defined(BSD) && BSD >= 199306)
   uint8_t                cur_min, cur_sec, cur_frame;
 #endif
 
@@ -177,7 +178,7 @@ static int input_vcd_read_toc (vcd_input_class_t *this, int fd) {
 
   return 0;
 }
-#elif defined (__FreeBSD_kernel__) || defined (__OpenBSD__)
+#elif defined (__FreeBSD_kernel__) || (defined(BSD) && BSD >= 199306)
 static int input_vcd_read_toc (vcd_input_class_t *this, int fd) {
 
   struct ioc_read_toc_entry te;
@@ -393,7 +394,7 @@ static off_t vcd_plugin_read (input_plugin_t *this_gen,
   memcpy (buf, data.data, VCDSECTORSIZE); /* FIXME */
   return VCDSECTORSIZE;
 }
-#elif defined (__FreeBSD_kernel__) || defined (__OpenBSD__)
+#elif defined (__FreeBSD_kernel__) || (defined(BSD) && BSD >= 199306)
 static off_t vcd_plugin_read (input_plugin_t *this_gen,
 				char *buf, off_t nlen) {
   vcd_input_plugin_t *this = (vcd_input_plugin_t *) this_gen;
@@ -531,7 +532,7 @@ static buf_element_t *vcd_plugin_read_block (input_plugin_t *this_gen,
   memcpy (buf->mem, data.data, VCDSECTORSIZE); /* FIXME */
   return buf;
 }
-#elif defined (__FreeBSD_kernel__) || defined (__OpenBSD__)
+#elif defined (__FreeBSD_kernel__) || (defined(BSD) && BSD >= 199306)
 static buf_element_t *vcd_plugin_read_block (input_plugin_t *this_gen,
 					     fifo_buffer_t *fifo, off_t nlen) {
 
@@ -690,7 +691,7 @@ static off_t vcd_plugin_seek (input_plugin_t *this_gen,
 
   return offset ; /* FIXME */
 }
-#elif defined (__FreeBSD_kernel__) || defined (__OpenBSD__)
+#elif defined (__FreeBSD_kernel__) || (defined(BSD) && BSD >= 199306)
 static off_t vcd_plugin_seek (input_plugin_t *this_gen,
 				off_t offset, int origin) {
 
@@ -764,7 +765,7 @@ static off_t vcd_plugin_get_length (input_plugin_t *this_gen) {
 
   return (off_t) 0;
 }
-#elif defined (__FreeBSD_kernel__) || defined (__OpenBSD__)
+#elif defined (__FreeBSD_kernel__) || (defined(BSD) && BSD >= 199306)
 static off_t vcd_plugin_get_length (input_plugin_t *this_gen) {
   vcd_input_plugin_t *this = (vcd_input_plugin_t *) this_gen;
   off_t len ;
@@ -828,7 +829,7 @@ static int vcd_plugin_open (input_plugin_t *this_gen) {
   char               *filename;
   int                 fd;
 
-  fd = open (cls->device, O_RDONLY|O_EXCL);
+  fd = xine_open_cloexec(cls->device, O_RDONLY|O_EXCL);
   if (fd == -1) {
     return 0;
   }
@@ -859,7 +860,7 @@ static int vcd_plugin_open (input_plugin_t *this_gen) {
   this->cur_min   = this->cls->tocent[this->cur_track].cdte_addr.msf.minute;
   this->cur_sec   = this->cls->tocent[this->cur_track].cdte_addr.msf.second;
   this->cur_frame = this->cls->tocent[this->cur_track].cdte_addr.msf.frame;
-#elif defined (__OpenBSD__)
+#elif defined (__OpenBSD__) || defined(__NetBSD__)
   this->cur_min   = this->cls->tocent[this->cur_track + 1 - this->cls->tochdr.starting_track].addr.msf.minute;
   this->cur_sec   = this->cls->tocent[this->cur_track + 1 - this->cls->tochdr.starting_track].addr.msf.second;
   this->cur_frame = this->cls->tocent[this->cur_track + 1 - this->cls->tochdr.starting_track].addr.msf.frame;
@@ -971,7 +972,7 @@ static xine_mrl_t **vcd_class_get_dir (input_class_t *this_gen, const char *file
     return NULL;
 
 
-  fd = open (this->device, O_RDONLY|O_EXCL);
+  fd = xine_open_cloexec(this->device, O_RDONLY|O_EXCL);
 
   if (fd == -1) {
     xprintf (this->xine, XINE_VERBOSITY_LOG,
@@ -1005,8 +1006,7 @@ static xine_mrl_t **vcd_class_get_dir (input_class_t *this_gen, const char *file
       memset(this->mrls[(i-1)], 0, sizeof(xine_mrl_t));
     }
 
-    asprintf(&(this->mrls[i-1]->mrl), "vcdo:/%d", i);
-
+    this->mrls[i-1]->mrl  = _x_asprintf("vcdo:/%d", i);
     this->mrls[i-1]->type = mrl_vcd;
 
     /* hack */
@@ -1033,7 +1033,7 @@ static char ** vcd_class_get_autoplay_list (input_class_t *this_gen, int *num_fi
   int i, fd;
 
 
-  fd = open (this->device, O_RDONLY|O_EXCL);
+  fd = xine_open_cloexec(this->device, O_RDONLY|O_EXCL);
 
   if (fd == -1) {
     xprintf (this->xine, XINE_VERBOSITY_LOG,
@@ -1060,7 +1060,7 @@ static char ** vcd_class_get_autoplay_list (input_class_t *this_gen, int *num_fi
 
   /* FIXME: check if track 0 contains valid data */
   for (i = 1; i < this->total_tracks; i++)
-    asprintf(&this->filelist[i-1], "vcdo:/%d", i);
+    this->filelist[i-1] = _x_asprintf("vcdo:/%d", i);
 
   /* printf ("%d tracks\n", this->total_tracks); */
 
