@@ -945,7 +945,11 @@ static void xvmc_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen) {
 
   if ( (frame->width != this->sc.delivered_width)
        || (frame->height != this->sc.delivered_height)
-       || (frame->ratio != this->sc.delivered_ratio) ) {
+       || (frame->ratio != this->sc.delivered_ratio)
+       || (frame->vo_frame.crop_left != this->sc.crop_left)
+       || (frame->vo_frame.crop_right != this->sc.crop_right)
+       || (frame->vo_frame.crop_top != this->sc.crop_top)
+       || (frame->vo_frame.crop_bottom != this->sc.crop_bottom) ) {
     lprintf("frame format changed\n");
 
     /*
