@@ -789,6 +789,14 @@ static const video_db_t video_db[] = {
   BUF_VIDEO_SNOW,
   "Snow"
 },
+{
+  {
+    ME_FOURCC('V','P','8','0'),
+    0
+  },
+  BUF_VIDEO_VP8,
+  "On2 VP8"
+},
 { { 0 }, 0, "last entry" }
 };
 
@@ -836,6 +844,7 @@ static const audio_db_t audio_db[] = {
     0x01,
     ME_FOURCC('r','a','w',' '),
     ME_FOURCC('s','o','w','t'),
+    ME_FOURCC('4','2','n','i'),
     0
   },
   BUF_AUDIO_LPCM_LE,
@@ -1183,6 +1192,14 @@ static const audio_db_t audio_db[] = {
   BUF_AUDIO_EAC3,
   "E-AC-3"
 },
+{
+  {
+    ME_FOURCC('M', 'P', '4', 'L'),
+    0
+  },
+  BUF_AUDIO_AAC_LATM,
+  "AAC LATM"
+},
 { { 0 }, 0, "last entry" }
 };
 
