@@ -371,7 +371,7 @@ static const char *config_translate_key (const char *key, char **tmp) {
   *tmp = NULL;
   if (!strncmp (key, "decoder.", 8) &&
       !strcmp (key + (trans = strlen (key)) - 9, "_priority")) {
-    asprintf (tmp, "engine.decoder_priorities.%.*s", trans - 17, key + 8);
+    *tmp = _x_asprintf ("engine.decoder_priorities.%.*s", trans - 17, key + 8);
     return *tmp;
   }
 
@@ -1012,8 +1012,8 @@ void xine_config_save (xine_t *xine, const char *filename) {
 
     lprintf("backing up configfile to %s\n", temp);
 
-    f_backup = fopen(temp, "w");
-    f_config = fopen(filename, "r");
+    f_backup = fopen(temp, "wb");
+    f_config = fopen(filename, "rb");
 
     if (f_config && f_backup && (stat(filename, &config_stat) == 0)) {
       char    *buf = NULL;
