@@ -654,6 +654,7 @@ xine_stream_t *xine_stream_new (xine_t *this,
   pthread_mutexattr_init(&attr);
   pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
   pthread_mutex_init (&stream->frontend_lock, &attr);
+  pthread_mutexattr_destroy(&attr);
 
   /*
    * Clear meta/stream info
@@ -1473,6 +1474,14 @@ static void xine_dispose_internal (xine_stream_t *stream) {
   xine_list_iterator_t *ite;
 
   lprintf("stream: %p\n", stream);
+
+  pthread_mutex_lock(&stream->xine->streams_lock);
+  ite = xine_list_find(stream->xine->streams, stream);
+  if (ite) {
+    xine_list_remove(stream->xine->streams, ite);
+  }
+  pthread_mutex_unlock(&stream->xine->streams_lock);
+
   pthread_mutex_destroy (&stream->info_mutex);
   pthread_mutex_destroy (&stream->meta_mutex);
   pthread_mutex_destroy (&stream->frontend_lock);
@@ -1488,12 +1497,7 @@ static void xine_dispose_internal (xine_stream_t *stream) {
 
   stream->metronom->exit (stream->metronom);
 
-  pthread_mutex_lock(&stream->xine->streams_lock);
-  ite = xine_list_find(stream->xine->streams, stream);
-  if (ite) {
-    xine_list_remove(stream->xine->streams, ite);
-  }
-  pthread_mutex_unlock(&stream->xine->streams_lock);
+  xine_list_delete(stream->event_queues);
 
   _x_refcounter_dispose(stream->refcounter);
 
@@ -1532,7 +1536,7 @@ void xine_dispose (xine_stream_t *stream) {
   if (stream->osd_renderer)
     stream->osd_renderer->close( stream->osd_renderer );
 
-
+  /* Remove the reference that the stream was created with. */
   _x_refcounter_dec(stream->refcounter);
 }
 
@@ -1561,6 +1565,8 @@ void xine_exit (xine_t *this) {
   if(this->port_ticket)
     this->port_ticket->dispose(this->port_ticket);
 
+  pthread_mutex_destroy(&this->log_lock);
+
 #if defined(WIN32)
   WSACleanup();
 #endif
@@ -1604,6 +1610,7 @@ xine_t *xine_new (void) {
    * log buffers
    */
   memset(this->log_buffers, 0, sizeof(this->log_buffers));
+  pthread_mutex_init (&this->log_lock, NULL);
 
 
 #ifdef WIN32
@@ -1665,7 +1672,7 @@ static void config_save_cb (void *this_gen, xine_cfg_entry_t *entry) {
   xine_t *this = (xine_t *)this_gen;
   char *homedir_trail_slash;
 
-  asprintf(&homedir_trail_slash, "%s/", xine_get_homedir());
+  homedir_trail_slash = _x_asprintf("%s/", xine_get_homedir());
   if (entry->str_value[0] &&
       (entry->str_value[0] != '/' || strstr(entry->str_value, "/.") ||
        strcmp(entry->str_value, xine_get_homedir()) == 0 ||
@@ -1696,7 +1703,6 @@ void xine_init (xine_t *this) {
    * locks
    */
   pthread_mutex_init (&this->streams_lock, NULL);
-  pthread_mutex_init (&this->log_lock, NULL);
 
   /* initialize color conversion tables and functions */
   init_yuv_conversion();
